using System;
using OpenTK.Mathematics;

namespace Dashboard.Media.Color
{
    public unsafe struct LockIO
    {
        public QImageLock Lock { get; }
        public int Width => Lock.Width;
        public int Height => Lock.Height;
        public int Depth => Depth;
        public QImageFormat Format => Lock.Format;

        public LockIO(QImageLock imageLock)
        {
            if (!imageLock.Format.IsU8())
                throw new Exception("Can only read/write U8 format images");

            Lock = imageLock;
        }

        public Color4 this[int index]
        {
            get
            {
                int chan = Format.Channels();
                byte *ptr = (byte*)Lock.ImagePtr + chan * index;

                switch (Format)
                {
                default:
                case QImageFormat.RedU8:    return new Color4(ptr[0], 0, 0, 255);
                case QImageFormat.AlphaU8:  return new Color4(0, 0, 0, ptr[0]);
                case QImageFormat.RaU8:     return new Color4(ptr[0], 0, 0, ptr[1]);
                case QImageFormat.RgbU8:    return new Color4(ptr[0], ptr[1], ptr[2], 255);
                case QImageFormat.RgbaU8:   return new Color4(ptr[0], ptr[1], ptr[2], ptr[3]);
                }
            }

            set
            {
                int chan = Format.Channels();
                byte *ptr = (byte*)Lock.ImagePtr + chan * index;

                switch (Format)
                {
                default:
                case QImageFormat.RedU8:
                    ptr[0] = (byte)(value.R * 255);
                    break;
                case QImageFormat.AlphaU8:
                    ptr[0] = (byte)(value.A * 255);
                    break;
                case QImageFormat.RaU8:
                    ptr[0] = (byte)(value.R * 255);
                    ptr[1] = (byte)(value.A * 255);
                    break;
                case QImageFormat.RgbU8:
                    ptr[0] = (byte)(value.R * 255);
                    ptr[1] = (byte)(value.G * 255);
                    ptr[2] = (byte)(value.B * 255);
                    break;
                case QImageFormat.RgbaU8:
                    ptr[0] = (byte)(value.R * 255);
                    ptr[1] = (byte)(value.G * 255);
                    ptr[2] = (byte)(value.B * 255);
                    ptr[3] = (byte)(value.A * 255);
                    break;
                }
            }
        }
        public Color4 this[int x, int y, int z = 0]
        {
            get => this[x + y * Width + z * Width * Height];
            set => this[x + y * Width + z * Width * Height] = value;
        }
    }

    public unsafe struct LockIOF
    {
        public QImageLock Lock { get; }
        public int Width => Lock.Width;
        public int Height => Lock.Height;
        public int Depth => Depth;
        public QImageFormat Format => Lock.Format;

        public LockIOF(QImageLock imageLock)
        {
            if (!imageLock.Format.IsFloat())
                throw new Exception("Can only read/write U8 format images");

            Lock = imageLock;
        }

        public Color4 this[int index]
        {
            get
            {
                int chan = Format.Channels();
                float *ptr = (float*)Lock.ImagePtr + chan * index;

                switch (Format)
                {
                default:
                case QImageFormat.RedU8:    return new Color4(ptr[0], 0, 0, 1);
                case QImageFormat.AlphaU8:  return new Color4(0, 0, 0, ptr[0]);
                case QImageFormat.RaU8:     return new Color4(ptr[0], 0, 0, ptr[1]);
                case QImageFormat.RgbU8:    return new Color4(ptr[0], ptr[1], ptr[2], 1);
                case QImageFormat.RgbaU8:   return new Color4(ptr[0], ptr[1], ptr[2], ptr[3]);
                }
            }

            set
            {
                int chan = Format.Channels();
                float *ptr = (float*)Lock.ImagePtr + chan * index;

                switch (Format)
                {
                default:
                case QImageFormat.RedU8:
                    ptr[0] = value.R;
                    break;
                case QImageFormat.AlphaU8:
                    ptr[0] = value.A;
                    break;
                case QImageFormat.RaU8:
                    ptr[0] = value.R;
                    ptr[1] = value.A;
                    break;
                case QImageFormat.RgbU8:
                    ptr[0] = value.R;
                    ptr[1] = value.G;
                    ptr[2] = value.B;
                    break;
                case QImageFormat.RgbaU8:
                    ptr[0] = value.R;
                    ptr[1] = value.G;
                    ptr[2] = value.B;
                    ptr[3] = value.A;
                    break;
                }
            }
        }
        public Color4 this[int x, int y, int z = 0]
        {
            get => this[x + y * Width + z * Width * Height];
            set => this[x + y * Width + z * Width * Height] = value;
        }
    }
}